#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os
import gettext
import logging
import time
import ssl
from collections import OrderedDict
from pathlib import Path
from PySide6.QtWidgets import (QMainWindow, QDialog)
from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QPixmap
# from PyQt5.QtWidgets import (QDialogButtonBox, QPushButton,
#                              QLineEdit, QHBoxLayout, QFrame, QLabel,
#                              QVBoxLayout, QStyle,)
# from PyQt5.QtCore import Qt
from http.client import HTTPConnection, HTTPSConnection
import orjson as json
from app.threads import VerifyConn
# from app.commons import connection
# from app.commons import common
from app.commons.config import Params
# from app.commons.dialogs import QuickDialog
# from app.commons.forms import GridForm
# from ..version import __version__
from app.commons.ui_login import Ui_Login
# from app.css.flat_button_small import *
context_http = ssl._create_unverified_context()

_ = gettext.gettext

__all__ = ['Login', 'xconnection']

pkg_dir = str(Path(os.path.dirname(__file__)).parents[0])
path_logo = os.path.join(pkg_dir, 'share', 'login.png')
path_circle_red = os.path.join(pkg_dir, 'share', 'circle_red.svg')
path_circle_green = os.path.join(pkg_dir, 'share', 'circle_green.svg')
file_base_css = os.path.join(pkg_dir, 'css', 'base.css')
file_tablet_css = os.path.join(pkg_dir, 'css', 'tablet.css')
HEADERS = {
    "Content-type": "application/json",
    "Accept": "text/plain"
}


class Login(QDialog):

    def __init__(self, parent=None, file_config=''):
        super(Login, self).__init__(parent)
        logging.info(' Start login Neox system X...')
        self.context = {}
        params = Params(file_config)
        self.params = params.params
        self.setObjectName('dialog_login')
        if self.params.get('tablet_mode') == 'True':
            self.tablet_mode = eval(self.params['tablet_mode'])
            self.set_style([file_tablet_css])
        else:
            if self.params.get('theme'):
                file_theme_css = os.path.join(
                    pkg_dir, 'css', self.params['theme'] + '.css')
                self.set_style([file_theme_css])
            else:
                self.set_style([file_base_css])
            self.tablet_mode = None
        self.ui = Ui_Login()
        self.ui.setupUi(self)
        self.verify_conn_th = VerifyConn(self)
        self.verify_conn_th.sigVerifyConn.connect(self.verify_active)
        self.verify_conn_th.start()

    def verify_active(self):
        conn = HTTPSConnection('google.com', timeout=5)
        try:
            conn.connect()
            conn._validate_host('google.com')
            conn.close()
            option = u"online"
            icon_conn = path_circle_green
        except Exception as e:
            print(e, 'error')
            icon_conn = path_circle_red
            option = u"offline"
        self.ui.label_conn.setText(option)
        self.ui.label_icon_conn.setPixmap(QPixmap(icon_conn))
        self.timer = QTimer()
        self.timer.timeout.connect(self.verify_active)
        self.timer.start(10)
        self.verify_conn_th.exit(0)

    def set_style(self, style_files):
        styles = []
        for style in style_files:
            with open(style, 'r') as infile:
                styles.append(infile.read())
        self.setStyleSheet(''.join(styles))

    def keyPressEvent(self, event):
        print(event, 'this is event')
        if event.key() == Qt.Key_Escape:
            self.reject()
        elif event.key() == Qt.Key_Enter:
            self.accept()
        event.accept()

    # def init_UI(self):
    #     hbox_logo = QHBoxLayout()
    #     label_logo = QLabel()
    #     label_logo.setObjectName('label_logo')
    #     hbox_logo.addWidget(label_logo, 0)
    #     pixmap_logo = QPixmap(path_logo)
    #     label_logo.setPixmap(pixmap_logo)
    #     hbox_logo.setAlignment(label_logo, Qt.AlignHCenter)

    #     values = OrderedDict([
    #         ('host', {'name': 'SERVIDOR', 'readonly': True}),
    #         ('database', {'name': 'BASE DE DATOS', 'readonly': True}),
    #         ('user', {'name': 'USUARIO'}),
    #         ('password', {'name': 'CONTRASEÑA'}),
    #     ])
    #     formLayout = GridForm(self, values=values, col=1)
    #     self.field_password.setEchoMode(QLineEdit.Password)
    #     self.field_password.textChanged.connect(self.clear_message)

    #     box_buttons = QDialogButtonBox()
    #     if self.params.get('option_offline') == 'True':
    #         pushButtonOffline = QPushButton("ONLINE")
    #         pushButtonOffline.setObjectName('button_offline')
    #         pushButtonOffline.setIcon(
    #             self.style().standardIcon(getattr(QStyle, 'SP_DriveNetIcon')))
    #         box_buttons.addButton(
    #             pushButtonOffline, QDialogButtonBox.ActionRole)
    #     pushButtonCancel = QPushButton("C&ANCELAR")
    #     pushButtonCancel.setObjectName('button_cancel')
    #     box_buttons.addButton(pushButtonCancel, QDialogButtonBox.RejectRole)
    #     pushButtonOk = QPushButton("&CONNECTAR")
    #     pushButtonOk.setAutoDefault(True)
    #     pushButtonOk.setDefault(False)
    #     pushButtonOk.setObjectName('button_ok')
    #     box_buttons.addButton(pushButtonOk, QDialogButtonBox.AcceptRole)

    #     hbox_buttons = QHBoxLayout()
    #     hbox_buttons.addWidget(box_buttons)

    #     line = QFrame()
    #     line.setFrameShape(line.HLine)
    #     line.setFrameShadow(line.Sunken)
    #     hbox_line = QHBoxLayout()
    #     hbox_line.addWidget(line)

    #     hbox_msg = QHBoxLayout()
    #     self.error_msg = QLabel('Error: usuario o contraseña invalida...!')
    #     self.error_msg.setObjectName('login_msg_error')
    #     self.error_msg.setAlignment(Qt.AlignCenter)
    #     hbox_msg.addWidget(self.error_msg)

    #     hbox_version = QHBoxLayout()
    #     label_version = QLabel('VERSION: ' + __version__)
    #     label_version.setMargin(20)
    #     font_version = QFont('Times')
    #     label_version.setFont(font_version)
    #     hbox_version.addWidget(label_version)
    #     hbox_version.setAlignment(Qt.AlignCenter)

    #     vbox_layout = QVBoxLayout()
    #     vbox_layout.addLayout(hbox_logo)
    #     vbox_layout.addLayout(formLayout)
    #     vbox_layout.addLayout(hbox_msg)
    #     vbox_layout.addLayout(hbox_line)
    #     vbox_layout.addLayout(hbox_buttons)
    #     vbox_layout.addLayout(hbox_version)

    #     self.setLayout(vbox_layout)
    #     self.setWindowTitle('Login Presik System')
    #     self.clear_message()

    #     self.field_password.setFocus()
    #     box_buttons.accepted.connect(self.accept)
    #     box_buttons.rejected.connect(self.reject)
    #     box_buttons.clicked.connect(self.clicked)

    def clear_message(self):
        self.error_msg.hide()

    def run(self, profile=None):
        # self.api_url = self.params['api_url']
        if self.params['database']:
            self.ui.field_database.setText(self.params['database'])
        if self.params['user']:
            self.ui.field_user.setText(self.params['user'])

    def accept(self):
        self.validate_access()
        super(Login, self).accept()

    def clicked(self, event):
        state = event.text()
        if state == 'ONLINE':
            event.setText('OFFLINE')
            event.setStyleSheet("background-color: rgb(255, 115, 0);")
            self.mode_conn = 'offline'
            self.error_msg.setText('Modo de facturacion offline activado...!')
            self.error_message()
            self.run()
        elif state == 'OFFLINE':
            event.setText('ONLINE')
            event.setStyleSheet("background-color: rgb(170, 175, 183);")
            self.mode_conn = 'online'
            self.clear_message()
            self.run()

    def reject(self):
        sys.exit()

    def validate_access(self):
        user = self.ui.field_user.text()
        password = self.ui.field_password.text()
        server = self.params.get('server')
        database = self.ui.field_database.text()
        result = xconnection(
            self.params['mode'], user, password, server, database, self.params['port']
        )
        if result['status'] != 200 or result['user'] is None:
            self.ui.field_password.setText('')
            self.ui.field_password.setFocus()
            self.run()
            msg = 'Error: usuario o contraseña invalida...!'
            if result['status'] == 429:
                msg = 'Error: Tiene muchos intentos de sesion, \npara desbloquear contacta con el area administrativa'
            elif result['status'] == 408:
                msg = 'Error: conexion del servidor'
            elif result['status'] == 500:
                msg = 'Error: interno del servidor \n' + result['message']
            print(msg, 'msg ,,,,', result['status'])
            self.ui.label_error.setText(msg)
            self.error_message()
        else:
            self.context = result
        self.params['user'] = user
        self.params['password'] = password

    def error_message(self):
        self.ui.label_error.show()


def xconnection(mode, user, password, host, database, port):
    # Get user_id and session
    if mode == 'http':
        conn = HTTPConnection(host, port=port, timeout=10)
    else:
        conn = HTTPSConnection(host, port=port, timeout=10, context=context_http)
    url = '/' + database + '/login'
    payload = {
        'method': "common.db.login",
        'params': [
            user,
            {'device_cookie': None, 'password': password},
            "es",
        ],
        'id': 0,
        'context': {},
    }
    try:
        conn.request('POST', url, body=json.dumps(payload), headers=HEADERS)
        response = conn.getresponse()
    except Exception as e:
        status = 408
        response = None
        print(e, 'error')

    # url = '%s://%s:%s@%s:%s/%s/' % (
    #     mode, user, password, host, 8000, database)
    try:
        res = json.loads(response.read())
        if not res.get('status'):
            res['status'] = response.status
    except:
        status = status
        msg = ''
        if response:
            status = response.status
            msg = response.msg

        res = {
            'status': status,
            'message': msg
        }

    conn.close()
    return res
    # return connection.set_xmlrpc(url)


# def safe_reconnect(main):
#     field_password = QLineEdit()
#     field_password.setEchoMode(QLineEdit.Password)
#     field_password.cursorPosition()
#     field_password.cursor()
#     dialog_password = QuickDialog(main, 'question',
#                                   info=main.tr('Enter your password:'),
#                                   widgets=[field_password],
#                                   buttons=['ok'],
#                                   response=True
#                                   )
#     field_password.setFocus()

#     password = field_password.text()
#     if not password or password == '':
#         safe_reconnect(main)

#     main.conn = xconnection(
#         main.user,
#         str(password),
#         main.server,
#         main.port,
#         main.database,
#         main.protocol,
#     )

#     if main.conn:
#         field_password.setText('')
#         dialog_password.hide()
#         main.global_timer = 0
#     else:
#         safe_reconnect(main)
